<?php

/*
Plugin Name: mobileadmin
Version: 1.1.1
Plugin URI: http://wordpress.org/extend/plugins/mobileadmin/
Description: Gives a mobile-friendly admin UI to browsers by User Agent. The iPhone/iPod-Touch are the primary supported devices for the first release.
Author: Jared Bangs and Dan Cameron
Author URI: http://wordpress.org/extend/plugins/mobileadmin/
*/ 

/*  Copyright 2007 Jared Bangs and Dan Cameron 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

class MobileAdminForm
{
	var $DebugMode = false;
	
	function FilterContent($content) { return $content; }

	function FilterContentCommon($content) { 
		
		if(!$this->IsRecentVersion()){
			
			// wp_admin_css filter is new in 2.3, so older versions will need to replace the CSS another way
			$selectingPattern = '/<link rel="stylesheet.*?\/>/mis';
			$csslink = '<link rel="stylesheet" href="' . get_bloginfo('wpurl') . '/' . PLUGINDIR .'/mobileadmin/css/iphone.css" type="text/css" media="screen" />';
			$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,$csslink);
			
			// jQuery is an older version than what we'd like, so replace it with 1.1.4
			$selectingPattern = '/wp-includes\/js\/jquery\//mis';
			$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,'');
			
		}
		return $content;
	}
	
	function FormatNavigationLink($text,$href) {
		return '<a href="' . $href . '">' . $text . '</a>';
	}

	function GetSelectFormPattern() {
		return "/<form[^>]*?method=['\"]post['\"].*?<\/form>/mis";
	}

	function GetSelectTopPattern() {
		return $this->GetSelectElementByIdPattern('div','user_info');
	}
	
	function IsRecentVersion() {
		return get_bloginfo('version') > '2.2.3';
	}
	
	function RenderDebugInfo() { 
		
		if($this->IsRecentVersion()){
			echo '<strong>Recent version </strong><pre>' . $this->IsRecentVersion() . '</pre>';
		}
		else {
			echo '<strong>Version </strong><pre>' . get_bloginfo('version') . '</pre>';
		}
		echo '<strong>REQUEST_URI</strong><pre>' . $_SERVER['REQUEST_URI'] . '</pre>';
		echo '<strong>HTTP_USER_AGENT</strong><pre>' . $_SERVER['HTTP_USER_AGENT'] . '</pre>';
		
		if(isset($_COOKIE['MobileAdminOverride'])) {
			echo '<strong>MobileAdminOverride Cookie</strong><pre>' . $_COOKIE['MobileAdminOverride'] . '</pre>';
		}
		else {
			echo '<strong>MobileAdminOverride Cookie</strong><pre> - not set</pre>';
		}
	}
	
	function RenderAdditionalFormData() { 
		if($this->DebugMode) {
			$this->RenderDebugInfo();
		}
	}

	function RenderCommonHeader() {
		global $user_identity;
		?>
		<meta name="viewport" content="maximum-scale=1.0,width=320,initial-scale=1.0" />
		<div id="mobilewphead"><a href="#"></a>
		<a href="<?php bloginfo('wpurl'); ?>/wp-admin/" title="Dashboard"><img class="dashboard" src="<?php bloginfo('wpurl') ?>/wp-content/plugins/mobileadmin/images/dashboard.png"></a>
		<a href="<?php bloginfo('url') ?>" title="Home"><img class="home" src="<?php bloginfo('wpurl') ?>/wp-content/plugins/mobileadmin/images/site.png"></a>
		<h1><a href="<?php bloginfo('url') . '/'; ?>" title="<?php bloginfo('description'); ?>"><?php bloginfo('name'); ?></a></h1>
		</div>
		<div id="mobilenav">
			<ul>
				<li><?php echo $this->FormatNavigationLink('Write','post-new.php'); ?></li>
				<li><?php echo $this->FormatNavigationLink('Comments','moderation.php'); ?></li>
				<li><?php echo $this->FormatNavigationLink('Posts','edit.php'); ?></li>
				<li><?php echo $this->FormatNavigationLink('Profile','profile.php'); ?></li>
			</ul>
			
		</div>
		<?php echo $this->RenderExtraHeaderInfo(); ?>
		<div id="ToggleMobileAdminView"></div>		
		
		<?php
	}

	function RenderExtraHeaderInfo() {	}
	
	function ReplaceDefaultHeader($content) {

		$topSelectPattern = $this->GetSelectTopPattern();

		// Move nav to the top
		$mobileNavSelectPattern = $this->GetSelectElementByIdPattern('div','mobilenav');
		$content = $this->MoveAfter($mobileNavSelectPattern,$topSelectPattern,$content);

		// Move nav to the top
		$toggleSelectPattern = $this->GetSelectElementByIdPattern('div','dashboard-top');
		$content = $this->MoveAfter($toggleSelectPattern,$mobileNavSelectPattern,$content);
		
		// Move custom header above nav, then delete current one
		$mobileHeaderSelectPattern = $this->GetSelectElementByIdPattern('div','mobilewphead');
		$content = $this->MoveBefore($mobileHeaderSelectPattern,$mobileNavSelectPattern,$content);
		$content = $this->StripWpHead($content);
		
		return $content;		
	}

	function StripOutAdminMenu($content) {
		$content = $this->StripOutElementById($content,'ul','adminmenu');
		return $content;		
	}

	function StripOutSubMenu($content) {
		$content = $this->StripOutElementById($content,'ul','submenu');
		return $content;		
	}

	function StripWpHead($content) { 
		$content = $this->StripOutElementById($content,'div','wphead');
		return $content;		
	}
	function StripUserInfo($content) { 
		$content = $this->StripOutElementById($content,'div','user_info');
		return $content;		
	}


	/* RegEx Utility Functions - Begin */
	function GetElementByClassName($content,$tagName,$className) {
		$selectingPattern = $this->GetSelectElementByClassNamePattern($tagName,$className);
		$matches = $this->GetMatches($selectingPattern,$content);
		
		// TODO: Bounds checking
		return $matches[0][0];
	}

	function GetElementById($content,$tagName,$id) {
		$selectingPattern = $this->GetSelectElementByIdPattern($tagName,$id);
		$matches = $this->GetMatches($selectingPattern,$content);
		
		// TODO: Bounds checking
		return $matches[0][0];
	}

	function GetMatches($pattern,$content) {
		preg_match_all($pattern, $content, $matches, PREG_SET_ORDER);
		return $matches;
	}

	function GetSelectElementByClassNamePattern($tagName,$className) {
		// TODO: fix this to support multiple class names
		return "/<". $tagName . "[^>]*?class=['\"]" . $className . "['\"].*?<\/" . $tagName . ">/mis";
	}

	function GetSelectElementByIdPattern($tagName,$id) {
		return "/<". $tagName . "[^>]*?id=['\"]" . $id . "['\"].*?<\/" . $tagName . ">/mis";
	}

	function MoveAfter($elementToMoveSelectingPattern,$moveAfterSelectingPattern,$content) {

		$elementToMoveMatches = $this->GetMatches($elementToMoveSelectingPattern,$content);
		$elementToMove = $elementToMoveMatches[0][0];
		
		$moveAfterMatches =  $this->GetMatches($moveAfterSelectingPattern,$content);
		$moveAfter = $moveAfterMatches[0][0];
		
		// Remove element from content
		$content = $this->ReplaceBySelectingPattern($content,$elementToMoveSelectingPattern, '');
		
		// Insert After
		$content = $this->ReplaceBySelectingPattern($content,$moveAfterSelectingPattern, $moveAfter . $elementToMove );
		
		return $content;
	}

	function MoveBefore($elementToMoveSelectingPattern,$moveBeforeSelectingPattern,$content) {

		$elementToMoveMatches = $this->GetMatches($elementToMoveSelectingPattern,$content);
		$elementToMove = $elementToMoveMatches[0][0];
		
		$moveBeforeMatches =  $this->GetMatches($moveBeforeSelectingPattern,$content);
		$moveBefore = $moveBeforeMatches[0][0];
		
		// Remove element from content
		$content = $this->ReplaceBySelectingPattern($content,$elementToMoveSelectingPattern, '');
		
		// Insert Before
		$content = $this->ReplaceBySelectingPattern($content,$moveBeforeSelectingPattern, $elementToMove . $moveBefore);
		
		return $content;
	}

	function ReplaceBySelectingPattern($content,$selectingPattern,$replacementValue) {
		$content = preg_replace($selectingPattern, $replacementValue, $content);
		return $content;		
	}

	function ReplaceElementByClassName($content,$tagName,$className,$replacementValue) {
		$selectingPattern = $this->GetSelectElementByClassNamePattern($tagName,$className);
		return $this->ReplaceBySelectingPattern($content,$selectingPattern, $replacementValue);
	}

	function ReplaceElementById($content,$tagName,$id,$replacementValue) {
		$selectingPattern = $this->GetSelectElementByIdPattern($tagName,$id);
		return $this->ReplaceBySelectingPattern($content,$selectingPattern, $replacementValue);
	}

	function StripOutElementById($content,$tagName,$id) {
		$replacementValue = "";
		return $this->ReplaceElementById($content,$tagName,$id,$replacementValue);
	}

	function StripOutElementByClassName($content,$tagName,$className) {
		$replacementValue = "";
		return $this->ReplaceElementByClassName($content,$tagName,$className,$replacementValue);
	}

	/* RegEx Utility Functions - End */

}

class MobileAdminCommentModerationForm extends MobileAdminForm
{
	function FilterContent($content) {
		$content = $this->FilterContentCommon($content);

		$content = $this->ReplaceDefaultHeader($content);

		return $content;
	}
	
	function FormatNavigationLink($text,$href) {
		if($text=="Comments") {
			return '<a class="activeNavLink" href="' . $href . '">' . $text . '</a>';
		}
		else {
			return parent::FormatNavigationLink($text,$href);
		}
	}
	
}


class MobileAdminDashboardForm extends MobileAdminForm
{
	function FilterContent($content) { 

		$content = $this->FilterContentCommon($content);

		$content = $this->ReplaceDefaultHeader($content);

		// Strip out uneccesary items
		$content = $this->StripOutAdminMenu($content);
		$content = $this->StripOutSubMenu($content);

		$content = $this->StripOutElementById($content,'div','devnews');
		$content = $this->StripOutElementById($content,'div','planetnews');
		
		// Strip out remaining end stuff from dashboard page
		$selectingPattern = "/<p>Use these links.*?forums<\/a>\.<\/p>/mis";
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,"");
		
		// Take out link to comments page in Comments heading
		$selectingPattern = "/<h3>Comments.*?<\/h3>/mis";
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,"<h3 class='ToggleLink'>Comments</h3>");
		
		// Take out link to comments page in Comments heading
		$selectingPattern = "/<h3>Posts.*?<\/h3>/mis";
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,"<h3 class='ToggleLink'>Posts</h3>");

		// Add class to Blog Stats heading
		$selectingPattern = "/<h3>Blog Stats.*?<\/h3>/mis";
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,"<h3 class='ToggleLink'>Blog Stats</h3>");

		// Create Incoming Links toggle
		$selectingPattern = $this->GetSelectElementByIdPattern('div','incominglinks');
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,"<h3 id='incominglinksToggle' class='ToggleLink'>Incoming Links</h3><div id='incominglinks'></div>");

		// Add class to div wrap
		$selectingPattern = '/<div class="wrap">/mis';
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,'<div class="dashboard wrap">');
		
		
		// Add toggle link to switch back to normal admin view
		$content = $this->ReplaceElementById($content,'div','ToggleMobileAdminView', 
			'<form method="post"><input type="submit" id="ToggleMobileAdminView" name="ToggleMobileAdminView" method="post" value="Revert to Normal Admin View" /></form>');

		
		return $content; 
	}	
	
	function RenderExtraHeaderInfo() {
		?>
		<div id="dashboard-top"><img class="logo" src="<?php echo get_bloginfo('wpurl') . '/' . PLUGINDIR .'/mobileadmin/images/logo.png'; ?>"></div>
		<?php
	}
	
}

class MobileAdminManageForm extends MobileAdminForm
{
	function DeTableizeMagagementList($content) {
		
		// Rows
		$output = '';
		$rowSelectingPattern = "/<tr.*?(class=?.*?|.*?)>.*?<\/tr>/mis";
		$rows = array();
		preg_match_all($rowSelectingPattern, $content, $rows);
		
		$rowCount = count($rows[0]); 
		
		$replacementOutput = "";
		for($i = 1; $i < $rowCount; $i++) {

			$rowContent = $rows[0][$i];
			$rowMetaData = $rows[1][$i];

			// Begin "row"
			$replacementOutput .= "<div".$rowMetaData.">";
			
			// Cells within row
			$cellSelectingPattern = "/<(th|td).*?(class=?.*?|.*?)>.*?<\/(th|td)>/mis";
			$cells = array();
			preg_match_all($cellSelectingPattern, $rowContent, $cells);
			
			// ID
			$replacementOutput .= "<span class=\"ID\">";
				$replacementOutput .= $this->GetCellInnerHTML($cells[0][0]);
			$replacementOutput .= "</span>";

			// PostName
			$replacementOutput .= "<span class=\"PostName\">";
				$replacementOutput .= $this->GetCellInnerHTML($cells[0][2]);
			$replacementOutput .= "</span>";

			// Comment Count
			$replacementOutput .= "<span class=\"Comments\">";
				$replacementOutput .= "(" . trim($this->GetCellInnerHTML($cells[0][4])) . " comments)";
			$replacementOutput .= "</span>";

			$replacementOutput .= "<br />";
			
			// View, Edit, Delete Links
			$replacementOutput .= "<span class=\"ManageActionLink View\">";
				$replacementOutput .= trim($this->GetCellInnerHTML($cells[0][6]));
			$replacementOutput .= "</span>";
			$replacementOutput .= "<span class=\"ManageActionLink Edit\">";
				$replacementOutput .= trim($this->GetCellInnerHTML($cells[0][7]));
			$replacementOutput .= "</span>";
						$replacementOutput .= "<span class=\"ManageActionLink Delete\">";
				$replacementOutput .= trim($this->GetCellInnerHTML($cells[0][8]));
			$replacementOutput .= "</span>";

			// End "row"
			$replacementOutput .= "</div>";
		}
		
		// Replace the table with the new content
		$selectingPattern = "/<table[^>]*?class=['\"]widefat['\"].*?>.*?<\/table>/mis";
		$replacementValue = "<div class=\"ManageTable\">". $replacementOutput . "</div>";
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,$replacementValue);
		
		return $content;
	}

	function GetCellInnerHTML($cellHTML) {
		$cellContentSelectingPattern = "/<(th|td).*?(class=?.*?|.*?)>(.*).*?<\/(th|td)>/mis";
		$cellContents = array();
		preg_match_all($cellContentSelectingPattern, $cellHTML, $cellContents);
		return $cellContents[3][0];
	}
	
	function FilterContent($content) {
		$content = $this->FilterContentCommon($content);

		$content = $this->ReplaceDefaultHeader($content);

		$content = $this->DeTableizeMagagementList($content);
		
		return $content;
	}
	
	function FormatNavigationLink($text,$href) {
		if($text=="Posts") {
			return '<a class="activeNavLink" href="' . $href . '">' . $text . '</a>';
		}
		else {
			return parent::FormatNavigationLink($text,$href);
		}
	}
}


class MobileAdminPostForm extends MobileAdminForm
{
	function FilterContent($content) { 
		
		$content = $this->FilterCommonPostContent($content);

		return $content; 
	}	

	function FilterCommonPostContent($content) { 
		
		$content = $this->FilterContentCommon($content);

		$content = $this->ReplaceDefaultHeader($content);
	
		$content = $this->StripOutElementById($content,'div','quicktags');

		$content = $this->StripUploadFrame($content);
		//$content = $this->StripPostExcerpt($content);
		$content = $this->StripTrackbacks($content);
		$content = $this->StripCustomFields($content);
				
		$content = $this->MovePostMetaItemsBelowPostText($content);

		return $content;
	}

	function FormatNavigationLink($text,$href) {
		if($text=="Write") {
			return '<a class="activeNavLink" href="' . $href . '">' . $text . '</a>';
		}
		else {
			return parent::FormatNavigationLink($text,$href);
		}
	}

	function MovePostMetaItemsBelowPostText($content) {
		$submitParagraphSelectPattern = $this->GetSelectElementByClassNamePattern('p','submit');

		// Post Status
		$poststatusSelectPattern = $this->GetSelectElementByIdPattern('fieldset','poststatusdiv');
		$content = $this->MoveBefore($poststatusSelectPattern,$submitParagraphSelectPattern,$content);

		// Categories
		$categorydivSelectPattern = $this->GetSelectElementByIdPattern('fieldset','categorydiv');
		$content = $this->MoveBefore($categorydivSelectPattern,$submitParagraphSelectPattern,$content);

		// Discussion
		$commentstatusdivSelectPattern = $this->GetSelectElementByIdPattern('fieldset','commentstatusdiv');
		$content = $this->MoveBefore($commentstatusdivSelectPattern,$submitParagraphSelectPattern,$content);

		// Post Slug
		$slugdivSelectPattern = $this->GetSelectElementByIdPattern('fieldset','slugdiv');
		$content = $this->MoveAfter($slugdivSelectPattern,$submitParagraphSelectPattern,$content);


		// Post Password
		$passworddivSelectPattern = $this->GetSelectElementByIdPattern('fieldset','passworddiv');
		$content = $this->MoveAfter($passworddivSelectPattern,$submitParagraphSelectPattern,$content);

		// Post Timestamp
		$posttimestampdivSelectPattern = "/<fieldset[^>]*?id=['\"]posttimestampdiv['\"].*?<\/fieldset>.*?<\/div>.*?<\/fieldset>/mis";
		$content = $this->MoveAfter($posttimestampdivSelectPattern,$submitParagraphSelectPattern,$content);


		return $content;		
	}

	function StripCustomFields($content) { 
		return $this->StripOutElementById($content,'fieldset','postcustom');
	}
	function StripPostExcerpt($content) { 
		return $this->StripOutElementById($content,'fieldset','postexcerpt');
	}
	function StripPostPassword($content) { 
		return $this->StripOutElementById($content,'fieldset','passworddiv');
	}
	function StripPostSlug($content) { 
		return $this->StripOutElementById($content,'fieldset','slugdiv');
	}
	function StripPostTimeStamp($content) { 

		// The Posttimestamp fieldset has a nested fieldset, which makes this a bit more complicated
		$selectingPattern = "/<fieldset[^>]*?id=['\"]posttimestampdiv['\"].*?<\/fieldset>.*?<\/div>.*?<\/fieldset>/mis";

		return $this->ReplaceBySelectingPattern($content,$selectingPattern,'');
	}
	function StripTrackbacks($content) { 
		return $this->StripOutElementById($content,'fieldset','trackbacksdiv');
	}
	function StripUploadFrame($content) { 
		return $this->StripOutElementById($content,'iframe','uploading');
	}
}

class MobileAdminPostNewForm extends MobileAdminPostForm
{
	function FilterContent($content) { 
		
		$content = $this->FilterCommonPostContent($content);

		return $content; 
	}	

}

class MobileAdminProfileForm extends MobileAdminForm
{
	function FilterContent($content) {
		$content = $this->FilterContentCommon($content);

		$content = $this->ReplaceDefaultHeader($content);

		// Strip out visual editor option and first update profile button
		$selectingPattern = "/<h3>Personal.*?name=\"submit\" \/><\/p>/mis";
		$content = $this->ReplaceBySelectingPattern($content,$selectingPattern,"");
		
		return $content;
	}
	
	function FormatNavigationLink($text,$href) {
		if($text=="Profile") {
			return '<a class="activeNavLink" href="' . $href . '">' . $text . '</a>';
		}
		else {
			return parent::FormatNavigationLink($text,$href);
		}
	}
	
}


function GetMobileAdminForm() {	
	
	$uri = $_SERVER['REQUEST_URI'];
	
	$isMobile = IsMobile();
	
	if( $isMobile && strpos($uri, 'wp-admin') != false ) {
		
		if( strpos($uri, 'wp-admin/')==(strlen($uri)-strlen('wp-admin/')) ) {
			return new MobileAdminDashboardForm();
		}
		else if( strpos($uri, 'wp-admin/index.php') != false ) {
			return new MobileAdminDashboardForm();
		}
		else if( strpos($uri, 'wp-admin/post-new.php') != false ) {
			return new MobileAdminPostNewForm();
		}
		else if( strpos($uri, 'wp-admin/post.php') != false ) {
			return new MobileAdminPostForm();
		}
		else if( strpos($uri, 'wp-admin/moderation.php') != false ) {
			return new MobileAdminCommentModerationForm();
		}
		else if( strpos($uri, 'wp-admin/edit.php') != false ) {
			return new MobileAdminManageForm();
		}
		else if( strpos($uri, 'wp-admin/profile.php') != false ) {
			return new MobileAdminProfileForm();
		}
		else {
			return new MobileAdminForm();
		}
	}
	else {
		return null;
	}
}

function MobileAdmin_RenderAdditionalFormData() {
	
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		$mobileAdminForm->RenderCommonHeader();
		$mobileAdminForm->RenderAdditionalFormData();
	}
}
add_action('admin_footer', 'MobileAdmin_RenderAdditionalFormData');

/* Mobile admin output content filter - Begin */
function mobileAdmin_RegExReplace($content) {
	
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		$content = $mobileAdminForm->FilterContent($content);
	}
	return $content;
}
function mobileAdmin_Initialize_RegExReplace() {
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		ob_start('mobileAdmin_RegExReplace');
	}
}
add_action('init', 'mobileAdmin_Initialize_RegExReplace', 11);
/* Mobile admin output content filter - End */

function MobileAdmin_FilterCSS($csslink) {
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		// TODO: *** Add CSS based on User Agent
		$csslink = '<link rel="stylesheet" href="' . get_bloginfo('wpurl') . '/' . PLUGINDIR .'/mobileadmin/css/iphone.css" type="text/css" media="screen" />';
	}
	return $csslink;
}
add_filter('wp_admin_css','MobileAdmin_FilterCSS');

function MobileAdmin_CustomScript() {
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		if(!$mobileAdminForm->IsRecentVersion()){
			echo '<script type="text/javascript" src="' . get_bloginfo('wpurl') . '/' . PLUGINDIR .'/mobileadmin/jquery/jquery.js"></script>';
			echo '<script type="text/javascript" src="' . get_bloginfo('wpurl') . '/' . PLUGINDIR .'/mobileadmin/jquery/interface.js"></script>';
		}
		echo '<script type="text/javascript" src="' . get_bloginfo('wpurl') . '/' . PLUGINDIR .'/mobileadmin/js/iphone.js"></script>';
	}
}
add_action('admin_head', 'MobileAdmin_CustomScript', 11);

function MobileAdmin_AlwaysIncludeJQuery() {
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		if($mobileAdminForm->IsRecentVersion()){
			wp_enqueue_script( 'interface' );
		}
	}
}
add_action('_admin_menu', 'MobileAdmin_AlwaysIncludeJQuery', 11);

function MobileAdmin_DisableRichEdit($wp_rich_edit) {
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm!=null) {
		$wp_rich_edit = false;
	}
	return $wp_rich_edit;
}
add_filter('user_can_richedit','MobileAdmin_DisableRichEdit');

function MobileAdmin_AddStandardUIMenu() {
	$mobileAdminForm = GetMobileAdminForm();
	if($mobileAdminForm==null) {
		add_options_page('Mobile Admin', 'Mobile Admin', 1, 'MobileAdminNormalUI', 'MobileAdmin_RenderStandardUIMenu');
	}
}
function MobileAdmin_RenderStandardUIMenu() {
	echo '<form method="post"><input style="font-size: 4em;padding: 10px 15px; margin: 30px;" type="submit" id="ToggleMobileAdminView" name="ToggleMobileAdminView" method="post" value="Switch to Mobile Admin View" /></form>';
}
add_action('admin_menu', 'MobileAdmin_AddStandardUIMenu');

function MobileAdmin_CheckOverrideCookie() {
	
	$overrideValue = false;
	
	if($_POST['ToggleMobileAdminView'] === 'Revert to Normal Admin View') {
		$overrideValue = true;
	}
	else if(isset($_COOKIE['MobileAdminOverride'])) {
		if($_COOKIE['MobileAdminOverride'] === 'true') {
			$overrideValue = true;
		}
	}
	
	return $overrideValue;
}
if($_POST['ToggleMobileAdminView'] === 'Revert to Normal Admin View') {
	setCookie('MobileAdminOverride','true');
}
else if($_POST['ToggleMobileAdminView'] === 'Switch to Mobile Admin View') {
	setCookie('MobileAdminOverride','false');
	header( 'Location: index.php' ) ;
}

/* IsMobile Check (borrowed from Alex King) - Begin */
function IsMobile() {
	
	// Only supports iPhone and iPod Touch
	
	if (!isset($_SERVER["HTTP_USER_AGENT"]) || MobileAdmin_CheckOverrideCookie()) {
		return false;
	}
	$whitelist = array(
		'Stand Alone/QNws'
	);
	foreach ($whitelist as $browser) {
		if (strstr($_SERVER["HTTP_USER_AGENT"], $browser)) {
			return false;
		}
	}
	
	// For debugging in Safari
	//update_option('MobileAdminDebugMode',true);
	if(get_option('MobileAdminDebugMode')) {
		$small_browsers = array(
			'iPhone','AppleWebKit','Mozilla'
		);
	}
	else {
		$small_browsers = array(
			'iPhone','iPod'
		);
	}

	foreach ($small_browsers as $browser) {
		if (strstr($_SERVER["HTTP_USER_AGENT"], $browser)) {
			return true;
		}
	}
	return false;
}
/* IsMobile Check (borrowed from Alex King) - End */

/* Workarounds for output buffer bug with PHP 5.2.x - Begin */
/* See PHP bug #39381 (http://bugs.php.net/bug.php?id=39381)
 * And WordPress Ticket#3354 (http://trac.wordpress.org/ticket/3354)
 * WP Trac seems to indicate that recent versions contain a fix for this
 * (Change 5463 - http://trac.wordpress.org/changeset/5463)
 * but I'm still seeing it in WPMU 1.2.1, so this is added just in case
 * people are using versions without the fix. It's a renamed copy of the 5463 change.
 */
if ( !function_exists('mobileAdmin_ob_end_flush_all') ) :
function mobileAdmin_ob_end_flush_all() 
{ 
	while ( @ob_end_flush() ); 
}
endif;
if (version_compare(phpversion(), '5.2.0', '>=')){
	add_action( 'shutdown', 'mobileAdmin_ob_end_flush_all', 1);
}
/* Workarounds for ob_start bug with PHP 5.2.x - End */
?>
